#!/usr/bin/perl
#
# $Id: DN_chooser.cgi,v 1.3 2000/09/09 13:47:51 seb Exp $
# This CGI generates the HTML for choosing one or a list of LDAP DN
#
#    OpenLDAP Webmin Module
#    Copyright (C) 2000 by S. Namche <snameche@virtual-net.fr>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#

use Net::LDAP;
use Net::LDAP::Util qw(ldap_error_name ldap_error_text);

require './web-lib.pl';
init_config();
ReadParse();

# <Temp>
$text{dns_title1}='Select DNs...';
$text{dns_title2}='Select DN...';
$text{dns_selection}='Selection';
$text{dns_ok}='Ok';
$text{dns_cancel}='Cancel';
$text{dns_clear}='Clear';
$text{dns_up}='up';
$text{dns_down}='down';
$sep = ',';
# </Temp>

sub uniformize_dn {
    $dn = shift;
    $dn =~ s/\s*,\s*/$sep/og;
    return $dn;
}

sub change_level {
    return "/DN_chooser.cgi?frame=1"        .
	   "&multi="  . $in{multi}          .
   	   "&user="   . urlize($in{user})   .
  	   "&base="   . urlize(uniformize_dn($_[0])) .
  	   "&host="   . urlize($in{host})   .
  	   "&port="   . urlize($in{port})   .
	   "&filter=" . urlize($in{filter}) ;
}


if( ! defined $in{frame} ){
    PrintHeader();

    if( $in{multi} ){
	my @dns = split( /\n+/, $in{user});
	print "<script>\n";
	print "sel = new Array(", scalar(@dns), ");\n";
	map { print "sel[$_] = \"$dns[$_]\";\n"  } (0..$#dns);
	print "</script>\n";
	print "<title>$text{dns_title1}</title>\n";
    }
    else {
	print "<script>\nsel = \"$in{user}\";\n</script>\n";
	print "<title>$text{dns_title2}</title>\n";
    }

    $level = change_level( $in{base} );
    print << "EOF";
<frameset rows='60%,40%'>
 <frame src="$level">
 <frameset cols='*,80' frameborder=no>
  <frame src="/DN_chooser.cgi?frame=2&multi=$in{multi}">
  <frame src="/DN_chooser.cgi?frame=3&multi=$in{multi}" scrolling=no>
 </frameset>
</frameset>
</body></html>
EOF
}

elsif( $in{frame} == 1 ){
    header();

    print "<script>\n";
    print "function adddn(u) {\n";
    print $in{multi} ? "top.sel[top.sel.length] = u;\n" : "top.sel = u;\n";
    print "top.frames[1].location = top.frames[1].location;\n";
    print "return false;\n";
    print "}\n";
    print "</script>\n";

    my $up = $in{base};
    $up =~ s/^[^,]+$sep(.+)$/$1/og;
    print "<table border=0 width=100%><tr><td width=10%><a href=\"", change_level($up), "\">$text{dns_up}</a></td>\n";
    print "<td><b>$in{base}</b></td></tr>\n";

    my $ldap = Net::LDAP->new( $in{host}, port => $in{port} ) or die( "new: $!" );
    $ldap->bind() or die( "bind: $!" );

    my $search = $ldap->search( base => $in{base}, scope => 'one', filter => $in{filter} );
    die( ldap_error_name($search->code()) . " (base: '$in{base}')" ) if $search->code();

    foreach my $entry ( $search->all_entries() ) {
	my $attr = uniformize_dn($entry->dn());
	$attr =~ s/$sep$in{base}//o;
	print "<tr><td><a href=\"", change_level($entry->dn()), "\">$text{dns_down}</a></td>\n<td>" ;
	print "<a href=\"\" onClick='return adddn(\"", uniformize_dn($entry->dn()), "\")'>$attr</a>";
	print "</td></tr>\n";
    }
    print "</table>\n";
}

elsif( $in{frame} == 2 ){
    header();

    print "<b>$text{dns_selection}</b>\n";

    print <<'EOF' if $in{multi};
<table width=100%>
<script>
  function sub(j) {
    sel2 = new Array();
    for(k=0,l=0; k<top.sel.length; k++) {
      if (k != j) {
	sel2[l] = top.sel[k];
	l++;
      }
    }
    top.sel = sel2;
    location = location;
    return false;
  }

  for(i=0; i<top.sel.length; i++) {
    document.write("<tr><td><a href=\"\" onClick='return sub("+i+")'>"+top.sel[i]+"</a></td></tr>\n");
  }
</script>
</table>
EOF

    print << 'EOF' if ! $in{multi};
<table width=100%>
<script>
  document.write("<tr><td>"+top.sel+"</td></tr>\n");
</script>
</table>
EOF
}

elsif( $in{'frame'} == 3 ){
    header();

    print << "EOF" if $in{multi};
<form>
 <center>
  <input type=button value="$text{dns_ok}" onClick='top.ifield.value=top.sel.join("\\n"); top.close()'><br>
  <input type=button value="$text{dns_cancel}" onClick='top.close()'><br>
  <input type=button value="$text{dns_clear}"
         onClick='top.sel = new Array(); top.frames[1].location = top.frames[1].location'>
 </center>
</form>
EOF

    print << "EOF" if ! $in{multi};
<form>
 <center>
  <input type=button value="$text{dns_ok}" onClick='top.ifield.value = top.sel; top.close()'><br>
  <input type=button value="$text{dns_cancel}" onClick='top.close()'>
 </center>
</form>
EOF
}
